@@ -1,12 +1,20 @@
+release_bullets <- function() {
+  c(
+    '`rhub::check_on_solaris(env_vars = c("_R_CHECK_FORCE_SUGGESTS_" = "false"))`',
+    '`rhub::check_with_valgrind(env_vars = c(VALGRIND_OPTS = "--leak-check=full --track-origins=yes"))`'
+  )
+}
+
 .onLoad <- function(...) {
   load_emoji_codes()
+  scan_local_fonts()
 }
 
 warn_env <- new.env(parent = emptyenv())
 warn_env$warned <- FALSE
 
 #' Get location of the fallback font
-#' 
+#'
 #' @export
 #' @keywords internal
 get_fallback <- function() {
